package com.qmth.themis.business.service.impl;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.bean.ExamPaperCacheBean;
import com.qmth.themis.business.dao.TEExamPaperMapper;
import com.qmth.themis.business.entity.TEExamPaper;
import com.qmth.themis.business.service.TEExamPaperService;
import com.qmth.themis.common.exception.BusinessException;

/**
 * @Description: 考试试卷 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TEExamPaperServiceImpl extends ServiceImpl<TEExamPaperMapper, TEExamPaper> implements TEExamPaperService {

	@Resource
	TEExamPaperMapper teExamPaperMapper;

	@Override
	public TEExamPaper findByExamIdAndCourseCodeAndPaperCode(Long examId, String courseCode, String paperCode) {
		if (examId == null) {
			throw new BusinessException("examId is null");
		}
		if (StringUtils.isBlank(courseCode)) {
			throw new BusinessException("courseCode is null");
		}
		if (StringUtils.isBlank(paperCode)) {
			throw new BusinessException("paperCode is null");
		}
		return teExamPaperMapper.findByExamIdAndCourseCodeAndPaperCode(examId, courseCode, paperCode);
	}
	
	@Cacheable(value = "exam_paper", key = "#paperId", unless = "#result == null")
	@Override
	public ExamPaperCacheBean getExamPaperCacheBean(Long paperId) {
		ExamPaperCacheBean ret = null;
		TEExamPaper ep = getById(paperId);
		if (ep == null) {
			return ret;
		}
		ret = new ExamPaperCacheBean();
		ret.setId(ep.getId());
		ret.setExamId(ep.getExamId());
		ret.setCourseCode(ep.getCourseCode());
		ret.setCode(ep.getCode());
		ret.setName(ep.getName());
		ret.setTotalScore(ep.getTotalScore());
		ret.setPaperPath(ep.getPaperPath());
		ret.setStructPath(ep.getStructPath());
		ret.setAnswerPath(ep.getAnswerPath());
		ret.setDecryptSecret(ep.getDecryptSecret());
		ret.setDecryptVector(ep.getDecryptVector());
		ret.setEncryptMode(ep.getEncryptMode());
		ret.setNeedVoiceAnswer(ep.getNeedVoiceAnswer());
		ret.setHasAudio(ep.getHasAudio());
		ret.setWeight(ep.getWeight());
		ret.setAudioPlayTime(ep.getAudioPlayTime());
		return ret;
	}
}
