package com.qmth.themis.mq.listener;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TMRocketMessage;
import com.qmth.themis.business.entity.TOeExamBreakHistory;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.enums.*;
import com.qmth.themis.business.service.TEExamStudentLogService;
import com.qmth.themis.business.service.TMRocketMessageService;
import com.qmth.themis.business.service.TOeExamBreakHistoryService;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.mq.dto.MqDto;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.client.consumer.listener.MessageListenerConcurrently;
import org.apache.rocketmq.common.consumer.ConsumeFromWhere;
import org.apache.rocketmq.common.message.Message;
import org.apache.rocketmq.common.message.MessageExt;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.annotation.SelectorType;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.apache.rocketmq.spring.core.RocketMQPushConsumerLifecycleListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 延时消息监听 websocket超时退出
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/2
 */
@Service
public class RocketWebsocketUnNormalLogConsumer implements MessageListenerConcurrently {
    private final static Logger log = LoggerFactory.getLogger(RocketWebsocketUnNormalLogConsumer.class);

    @Resource
    RedisUtil redisUtil;

    @Resource
    TEExamStudentLogService teExamStudentLogService;

    @Resource
    TMRocketMessageService tmRocketMessageService;

    @Resource
    TOeExamRecordService tOeExamRecordService;

    @Resource
    TOeExamBreakHistoryService tOeExamBreakHistoryService;

    @Override
    @Transactional
    public ConsumeConcurrentlyStatus consumeMessage(List<MessageExt> msgs, ConsumeConcurrentlyContext consumeConcurrentlyContext) {
        MqDto mqDto = null;
        TOeExamRecord tOeExamRecord = null;
        TOeExamBreakHistory tOeExamBreakHistory = null;
        TOeExamRecord tOeExamRecordUpdate = null;
        try {
            long threadId = Thread.currentThread().getId();
            String threadName = Thread.currentThread().getName();
            Gson gson = new Gson();
            for (MessageExt messageExt : msgs) {
                log.info(":{}-:{} websocket un normal logConsumer重试次数:{}", threadId, threadName, messageExt.getReconsumeTimes());
                String body = new String(messageExt.getBody(), Constants.CHARSET_NAME);
                log.info("body:{}", body);
                JSONObject jsonObject = JSONObject.parseObject(body);
                Map properties = (Map) jsonObject.get("properties");
                mqDto = JSONObject.toJavaObject(JSONObject.parseObject(String.valueOf(properties.get(SystemConstant.MQDTO_OBJ))), MqDto.class);
                int reconsumeTime = messageExt.getReconsumeTimes();
                if (reconsumeTime >= SystemConstant.MAXRECONSUMETIMES) {
                    //超过最大重试次数，保存到数据库，后续可以发短信通知系统管理人员
                    mqDto.setAck(SystemConstant.POSION_ACK_TYPE);
                    TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
                    tmRocketMessage.setProp(JacksonUtil.parseJson(tmRocketMessage.getProperties()));
                    tmRocketMessageService.saveOrUpdate(tmRocketMessage);
                    redisUtil.delete(SystemConstant.WEBSOCKET_UN_NORMAL_LOG_TOPIC_BUFFER_LIST, mqDto.getId());
                } else {
                    if (Objects.nonNull(mqDto.getAck()) && mqDto.getAck().intValue() != SystemConstant.STANDARD_ACK_TYPE && Objects.nonNull(SystemConstant.WEBSOCKET_UN_NORMAL_LOG_TOPIC_BUFFER_LIST) && redisUtil.lock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId(), SystemConstant.REDIS_LOCK_MQ_TIME_OUT)) {
                        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);//表示成功处理
                        Map<String, Object> tranMap = mqDto.getProperties();
                        Long recordId = Long.parseLong(String.valueOf(tranMap.get("recordId")));
                        String deviceId = String.valueOf(tranMap.get("deviceId"));
                        String ip = String.valueOf(tranMap.get("ip"));
                        Long updateTime = Long.parseLong(String.valueOf(tranMap.get("updateTime")));
                        Date date = new Date();
                        date.setTime(updateTime);
                        tOeExamRecord = tOeExamRecordService.getById(recordId);
                        tOeExamRecordUpdate = new TOeExamRecord();
                        BeanUtils.copyProperties(tOeExamRecord, tOeExamRecordUpdate);
                        Integer breakCount = tOeExamRecord.getLeftBreakResumeCount();
                        if (Objects.isNull(breakCount) || breakCount <= 0) {
                            //todo 没有断点次数，则强制交卷
                            tOeExamRecord.setStatus(ExamRecordStatusEnum.finished.ordinal());
                            tOeExamRecordService.updateById(tOeExamRecord);
                        } else {
                            breakCount--;
                            //增加断点记录
                            tOeExamBreakHistory = new TOeExamBreakHistory(recordId, new Date(), WebsocketExceptionEnum.NET_TIME_OUT.ordinal(), WebsocketExceptionEnum.NET_TIME_OUT.name());
                            tOeExamBreakHistoryService.save(tOeExamBreakHistory);
                            //更新考试记录状态
                            tOeExamRecord.setClientCurrentIp(ip);
                            tOeExamRecord.setClientWebsocketId(deviceId);
                            tOeExamRecord.setClientWebsocketStatus(WebsocketStatusEnum.UN_ONLINE.ordinal());
                            tOeExamRecord.setClientLastSyncTime(date);
                            tOeExamRecord.setLastBreakId(tOeExamBreakHistory.getId());
                            tOeExamRecord.setLastBreakTime(tOeExamBreakHistory.getBreakTime());
                            tOeExamRecord.setLeftBreakResumeCount(breakCount);
                            tOeExamRecord.setStatus(ExamRecordStatusEnum.break_off.ordinal());
                            tOeExamRecordService.updateById(tOeExamRecord);
                        }
                    } else {
                        mqDto.setAck(SystemConstant.INDIVIDUAL_ACK_TYPE);//表示考生已重新登录
                    }
                    log.info(":{}-:{} 插入用户轨迹日志", threadId, threadName);
                    teExamStudentLogService.saveStudentLogInfo(mqDto.getTimestamp(), MqEnum.valueOf(String.valueOf(mqDto.getType())).getId(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), JacksonUtil.parseJson(mqDto));
                    TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
                    tmRocketMessage.setProp(JacksonUtil.parseJson(tmRocketMessage.getProperties()));
                    tmRocketMessageService.saveOrUpdate(tmRocketMessage);
                    redisUtil.delete(SystemConstant.WEBSOCKET_UN_NORMAL_LOG_TOPIC_BUFFER_LIST, mqDto.getId());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            if (Objects.nonNull(tOeExamRecordUpdate)) {
                tOeExamRecordService.updateById(tOeExamRecordUpdate);
            }
            if (Objects.nonNull(tOeExamBreakHistory)) {
                tOeExamBreakHistoryService.removeById(tOeExamBreakHistory.getId());
            }
            return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
        } finally {
            if (Objects.nonNull(mqDto)) {
                redisUtil.releaseLock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId());
            }
        }
        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;//成功
    }

    @Service
    @RocketMQMessageListener(consumerGroup = "${mq.config.websocketUnNormalConsumerOeGroup}", topic = "${mq.config.websocketUnNormalTopic}", selectorType = SelectorType.TAG, selectorExpression = "${mq.config.websocketUnNormalTopicOeTag}")
    public class sessionConsumerOeLog implements RocketMQListener<Message>, RocketMQPushConsumerLifecycleListener {

        @Override
        public void onMessage(Message message) {
            //实现RocketMQPushConsumerLifecycleListener监听器之后，此方法不调用
        }

        @Override
        public void prepareStart(DefaultMQPushConsumer defaultMQPushConsumer) {
            defaultMQPushConsumer.setConsumeMessageBatchMaxSize(SystemConstant.CONSUME_MESSAGE_BATCH_MAX_SIZE);//每次拉取10条
            defaultMQPushConsumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_FIRST_OFFSET);
            defaultMQPushConsumer.setMaxReconsumeTimes(SystemConstant.MAXRECONSUMETIMES);//最大重试次数
            defaultMQPushConsumer.registerMessageListener(RocketWebsocketUnNormalLogConsumer.this::consumeMessage);
        }
    }

//    @Service
//    @RocketMQMessageListener(consumerGroup = "${mq.config.userLogConsumerStudentGroup}", topic = "${mq.config.userLogTopic}", selectorType = SelectorType.TAG, selectorExpression = "${mq.config.userLogTopicStudentTag}")
//    public class sessionConsumerStudentLog implements RocketMQListener<Message>, RocketMQPushConsumerLifecycleListener {
//
//        @Override
//        public void onMessage(Message message) {
//            //实现RocketMQPushConsumerLifecycleListener监听器之后，此方法不调用
//        }
//
//        @Override
//        public void prepareStart(DefaultMQPushConsumer defaultMQPushConsumer) {
//            defaultMQPushConsumer.setConsumeMessageBatchMaxSize(SystemConstant.CONSUME_MESSAGE_BATCH_MAX_SIZE);//每次拉取10条
//            defaultMQPushConsumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_FIRST_OFFSET);
//            defaultMQPushConsumer.setMaxReconsumeTimes(SystemConstant.MAXRECONSUMETIMES);//最大重试次数
//            defaultMQPushConsumer.registerMessageListener(RocketWebsocketOeLogConsumer.this::consumeMessage);
//        }
//    }
}
